package sim.util;

import java.awt.*;

public class LightContainer extends Container
{
	public LightContainer()
	{
		super();
	}
	
	public LightContainer(LayoutManager layout)
	{
		super();
		this.setLayout(layout);
	}
	
//	public void paint(Graphics g)
//	{
//		g.setColor(this.getBackground());
//		g.fillRect(0, 0, this.getSize().width, this.getSize().height);
//		
//		super.paint(g);
//	}
	
	public Dimension getPreferredSize()
	{
		return this.getSize();
	}
}
