package sim.lib.gates;

import java.awt.Image;

import sim.GuiFileLink;

public class GateNOR extends GateOR
{
	private static Image ICON = GuiFileLink.getImage("sim/lib/gates/NorIcon.gif");
	
	public Image getIcon()
	{
		return GateNOR.ICON;
	}
	
	public String getBubbleHelp()
	{
		return "NOR gate";
	}
	
	public GateNOR()
	{
		super();
		this.setOutputInvert(true);
	}
}