package sim.lib.gates;

import java.awt.Image;

import sim.GuiFileLink;

public class GateNAND extends GateAND
{
	private static Image ICON = GuiFileLink.getImage("sim/lib/gates/NandIcon.gif");
	
	public Image getIcon()
	{
		return GateNAND.ICON;
	}
	
	public String getBubbleHelp()
	{
		return "NAND gate";
	}
	
	public GateNAND()
	{
		super();
		this.setOutputInvert(true);
	}
}
